/* This file is automatically generated from cssproperties.in by makeprop, do not edit */
/* Copyright 1998 W. Bastian */

#ifndef CSSPROPERTIES_H
#define CSSPROPERTIES_H


#define CSS_PROP_INVALID 0
#define CSS_PROP_MIN 1
#define CSS_PROP_BACKGROUND_COLOR 1
#define CSS_PROP_BACKGROUND_IMAGE 2
#define CSS_PROP_BACKGROUND_REPEAT 3
#define CSS_PROP_BACKGROUND_ATTACHMENT 4
#define CSS_PROP_BACKGROUND_POSITION 5
#define CSS_PROP_BACKGROUND_POSITION_X 6
#define CSS_PROP_BACKGROUND_POSITION_Y 7
#define CSS_PROP_BORDER_COLLAPSE 8
#define CSS_PROP_BORDER_SPACING 9
#define CSS_PROP_BORDER_TOP_COLOR 10
#define CSS_PROP_BORDER_RIGHT_COLOR 11
#define CSS_PROP_BORDER_BOTTOM_COLOR 12
#define CSS_PROP_BORDER_LEFT_COLOR 13
#define CSS_PROP_BORDER_TOP_STYLE 14
#define CSS_PROP_BORDER_RIGHT_STYLE 15
#define CSS_PROP_BORDER_BOTTOM_STYLE 16
#define CSS_PROP_BORDER_LEFT_STYLE 17
#define CSS_PROP_BORDER_TOP_WIDTH 18
#define CSS_PROP_BORDER_RIGHT_WIDTH 19
#define CSS_PROP_BORDER_BOTTOM_WIDTH 20
#define CSS_PROP_BORDER_LEFT_WIDTH 21
#define CSS_PROP_BOTTOM 22
#define CSS_PROP_CAPTION_SIDE 23
#define CSS_PROP_CLEAR 24
#define CSS_PROP_CLIP 25
#define CSS_PROP_COLOR 26
#define CSS_PROP_CONTENT 27
#define CSS_PROP_COUNTER_INCREMENT 28
#define CSS_PROP_COUNTER_RESET 29
#define CSS_PROP_CURSOR 30
#define CSS_PROP_DIRECTION 31
#define CSS_PROP_DISPLAY 32
#define CSS_PROP_EMPTY_CELLS 33
#define CSS_PROP_FLOAT 34
#define CSS_PROP_FONT_FAMILY 35
#define CSS_PROP_FONT_SIZE 36
#define CSS_PROP_FONT_SIZE_ADJUST 37
#define CSS_PROP_FONT_STRETCH 38
#define CSS_PROP_FONT_STYLE 39
#define CSS_PROP_FONT_VARIANT 40
#define CSS_PROP_FONT_WEIGHT 41
#define CSS_PROP_HEIGHT 42
#define CSS_PROP_LEFT 43
#define CSS_PROP_LETTER_SPACING 44
#define CSS_PROP_LINE_HEIGHT 45
#define CSS_PROP_LIST_STYLE_IMAGE 46
#define CSS_PROP_LIST_STYLE_POSITION 47
#define CSS_PROP_LIST_STYLE_TYPE 48
#define CSS_PROP_MARGIN_TOP 49
#define CSS_PROP_MARGIN_RIGHT 50
#define CSS_PROP_MARGIN_BOTTOM 51
#define CSS_PROP_MARGIN_LEFT 52
#define CSS_PROP_MARKER_OFFSET 53
#define CSS_PROP_MAX_HEIGHT 54
#define CSS_PROP_MAX_WIDTH 55
#define CSS_PROP_MIN_HEIGHT 56
#define CSS_PROP_MIN_WIDTH 57
#define CSS_PROP_ORPHANS 58
#define CSS_PROP_OUTLINE_COLOR 59
#define CSS_PROP_OUTLINE_STYLE 60
#define CSS_PROP_OUTLINE_WIDTH 61
#define CSS_PROP_OVERFLOW 62
#define CSS_PROP_PADDING_TOP 63
#define CSS_PROP_PADDING_RIGHT 64
#define CSS_PROP_PADDING_BOTTOM 65
#define CSS_PROP_PADDING_LEFT 66
#define CSS_PROP_PAGE 67
#define CSS_PROP_PAGE_BREAK_AFTER 68
#define CSS_PROP_PAGE_BREAK_BEFORE 69
#define CSS_PROP_PAGE_BREAK_INSIDE 70
#define CSS_PROP_POSITION 71
#define CSS_PROP_QUOTES 72
#define CSS_PROP_RIGHT 73
#define CSS_PROP_SIZE 74
#define CSS_PROP_TABLE_LAYOUT 75
#define CSS_PROP_TEXT_ALIGN 76
#define CSS_PROP_TEXT_DECORATION 77
#define CSS_PROP_TEXT_DECORATION_COLOR 78
#define CSS_PROP_TEXT_INDENT 79
#define CSS_PROP_TEXT_SHADOW 80
#define CSS_PROP_TEXT_TRANSFORM 81
#define CSS_PROP_TOP 82
#define CSS_PROP_UNICODE_BIDI 83
#define CSS_PROP_VERTICAL_ALIGN 84
#define CSS_PROP_VISIBILITY 85
#define CSS_PROP_WHITE_SPACE 86
#define CSS_PROP_WIDOWS 87
#define CSS_PROP_WIDTH 88
#define CSS_PROP_WORD_SPACING 89
#define CSS_PROP_Z_INDEX 90
#define CSS_PROP_BACKGROUND 91
#define CSS_PROP_BORDER 92
#define CSS_PROP_BORDER_COLOR 93
#define CSS_PROP_BORDER_STYLE 94
#define CSS_PROP_BORDER_TOP 95
#define CSS_PROP_BORDER_RIGHT 96
#define CSS_PROP_BORDER_BOTTOM 97
#define CSS_PROP_BORDER_LEFT 98
#define CSS_PROP_BORDER_WIDTH 99
#define CSS_PROP_FONT 100
#define CSS_PROP_LIST_STYLE 101
#define CSS_PROP_MARGIN 102
#define CSS_PROP_OUTLINE 103
#define CSS_PROP_PADDING 104
#define CSS_PROP_SCROLLBAR_FACE_COLOR 105
#define CSS_PROP_SCROLLBAR_SHADOW_COLOR 106
#define CSS_PROP_SCROLLBAR_HIGHLIGHT_COLOR 107
#define CSS_PROP_SCROLLBAR_3DLIGHT_COLOR 108
#define CSS_PROP_SCROLLBAR_DARKSHADOW_COLOR 109
#define CSS_PROP_SCROLLBAR_TRACK_COLOR 110
#define CSS_PROP_SCROLLBAR_ARROW_COLOR 111
#define CSS_PROP__KONQ_FLOW_MODE 112

#define CSS_PROP_MAX CSS_PROP_Z_INDEX
#define CSS_PROP_TOTAL 113
#endif

